@@ -4,12 +4,13 @@ use serde::{Deserialize, Serialize};
 use uuid::Uuid;
 
 use crate::context::Context;
+use crate::customer::{Address, Contact};
 use crate::error::{new_application_error, new_bad_request_error, Error};
 use crate::inventory::ReserveResult;
 use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
-use crate::shipping::{Address, FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
+use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
 
 #[derive(Serialize, Deserialize, JsonSchema)]
 pub enum State {
@@ -25,8 +26,9 @@ pub struct Checkout {
     pub currency: Currency,
     pub promo_codes: Vec<String>,
     pub items: Vec<Item>,
-    pub fulfillment_type: Option<FulfillmentType>,
+    pub contact: Option<Contact>,
     pub shipping_address: Option<Address>,
+    pub fulfillment_type: Option<FulfillmentType>,
     pub shipping_quotes: Vec<ShippingQuote>,
     pub invoice: Option<Invoice>,
 }
@@ -121,15 +123,21 @@ impl Checkout {
 
 // public methods
 impl Checkout {
-    pub async fn create<C: Context + Send>(ctx: &mut C, currency: Currency) -> Result<Self, Error> {
+    pub async fn create<C: Context + Send>(
+        ctx: &mut C,
+        currency: Currency,
+        contact: Option<Contact>,
+        shipping_address: Option<Address>,
+    ) -> Result<Self, Error> {
         let checkout = Checkout {
             id: Uuid::new_v4(),
             state: State::Shopping,
             currency,
             promo_codes: vec![],
             items: vec![],
+            contact,
+            shipping_address,
             fulfillment_type: None,
-            shipping_address: None,
             shipping_quotes: vec![],
             invoice: None,
         };
@@ -225,6 +233,15 @@ impl Checkout {
         Ok(ctx.on_confirm_items(self).await?)
     }
 
+    pub async fn update_contact<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+        contact: Contact,
+    ) -> Result<(), Error> {
+        self.contact = Some(contact);
+        ctx.on_update_contact(self).await
+    }
+
     pub async fn update_shipping_address<C: Context + Send>(
         &mut self,
         ctx: &mut C,
